/*************************************************************************************
Data Visualization - KnowWhereGraph Statement Rain
Tested in Processing 4.0b3                                    
                                                     
Author: Zilong Liu                                       

References: Parts of the codes were developed based upon Simple 3D Rain Simulation
            by humayung.
            See: https://discourse.processing.org/t/simple-3d-rain-simulation/10834

Usage: 1. To use droplets to show hazard-related statements with explicit temporal 
          scopes in KnowWhereGraph.
       2. To use splashes to show the spatial information of those statements.
*************************************************************************************/
import peasy.*;
import controlP5.*;
import java.util.*;

PeasyCam cam;
ControlP5 cp5;
PFont mono;

int resolution = 1;
RainSystem rs;
SplashSystem ss;
String kwg_prefix = "http://stko-kwg.geog.ucsb.edu/lod/ontology/";
int counter_draw = 0;
int counter_earthquake = 0;
int counter_wildfire = 0;
int counter_hurricane = 0;
String currentDateTime;

void setup() {
  fullScreen(P3D);
  cam = new PeasyCam(this, 600);
  cp5 = new ControlP5(this);
  mono = loadFont("courier.vlw");   // load the courier font style
  textFont(mono);
  readQueryResults(); // read query results
  rs = new RainSystem();
  ss = new SplashSystem();
}

void draw() {
  background(#0D0208);
  textMode(SHAPE);
  colorMode(RGB);
  rotateX(-0.3);
  rotateY(0.1);
  rotateZ(-1.2);
  // ortho(); // Set an orthographic projection

  //pouring
  if ((counter_draw+1)*resolution >= hazardStatement_list.size())
  {
    currentDateTime = hazardStatement_list.get(counter_draw*resolution).start_datetime;
    rs.run(hazardStatement_list.subList(counter_draw*resolution, hazardStatement_list.size()-1));
    ss.run();
    counter_draw = 0;
    counter_earthquake = 0;
    counter_hurricane = 0;
    counter_wildfire = 0;
    rs = new RainSystem();
    ss = new SplashSystem();
  }
  else
  {
    currentDateTime = hazardStatement_list.get(counter_draw*resolution).start_datetime;
    rs.run(hazardStatement_list.subList(counter_draw*resolution, (counter_draw+1)*resolution));
    ss.run();
    counter_draw += 1;
  }
  
  draw_project_info();
}
