class Hazard{
  String hazard;
  String hazard_label;
  String hazard_type;
  String time;
  String start_datetime;
  float interval;
  String location;
  String location_label;

  Hazard(String hazard, String hazard_label, String hazard_type, 
  String time, String time_label, String location, String location_label)
  {
    this.hazard = hazard;
    this.hazard_label = hazard_label;
    this.hazard_type = hazard_type;
    this.time = time;
    Random random = new Random();
    if (hazard_type.equals(kwg_prefix+"Hurricane"))
    {
      if (time_label.length() > 19) // no end datetime
      {
        String start_datetime = time_label.substring(0,10);
        this.start_datetime = start_datetime;
        //String end_datetime = time_label.substring(19,34);
        this.interval = (random.nextFloat(13) + 12)*60; // https://www.cheatsheet.com/culture/how-long-do-hurricanes-last-on-average.html/
      }
      else
      {
        String start_datetime = time_label.substring(0,10);
        this.start_datetime = start_datetime;
        this.interval = (random.nextFloat(13) + 12)*60; // https://www.cheatsheet.com/culture/how-long-do-hurricanes-last-on-average.html/
      }
    }
    else if (hazard_type.equals(kwg_prefix+"Wildfire"))
    {
      this.start_datetime = time_label.substring(0,10);
      this.interval = 37*24*60; // https://facethefactsusa.org/facts/the-national-burn-rate-is-going-up--literally/
    }
    else if (hazard_type.equals(kwg_prefix+"EarthquakeEvent"))
    {
      this.start_datetime = time_label.substring(0,10);
      this.interval = random.nextFloat(5) + 1; // https://www.gns.cri.nz/Home/Learning/Science-Topics/Earthquakes/Monitoring-Earthquakes/Other-earthquake-questions/How-long-does-an-earthquake-last#:~:text=During%20the%20very%20largest%20earthquakes,be%20felt%20for%20some%20hours.
    }
    this.location = location;
    this.location_label = location_label;
  }

  String getStartDateTime()
  {
    return this.start_datetime;
  }
}

class Droplet{
  PVector pos; // Keep track droplet position
  PVector vel; // droplet velocity
  boolean off; // hit the ground, and die
  Hazard hazard;
  PVector gravity;

  Droplet(float x, float y, float z, Hazard hazard){
    pos = new PVector(x,y,z);
    vel = new PVector();
    this.hazard = hazard;
    if (this.hazard.hazard_type.equals(kwg_prefix+"EarthquakeEvent"))
    {
      counter_earthquake += 1;
      this.gravity = new PVector(0, map(1/hazard.interval, (float) 1/5, (float) 1, 0.9, 1.2), 0);
    }
    else if (this.hazard.hazard_type.equals(kwg_prefix+"Wildfire"))
    {
      counter_wildfire += 1;
      this.gravity = new PVector(0, 0.1, 0);
    }
    else if (this.hazard.hazard_type.equals(kwg_prefix+"Hurricane"))
    {
      counter_hurricane += 1;
      this.gravity = new PVector(0, map(1/hazard.interval, (float) 1/(24*60), (float) 1/(12*60), 0.2, 0.5), 0);
    }
  }
  
  void show(){
    String statement = this.hazard.hazard_label;
    String hazardType = this.hazard.hazard_type;
    strokeWeight(1.5);    
    pushMatrix();
    translate(pos.x, pos.y, pos.z);
    rotate(HALF_PI);
    int fontSize = 30;

    int colorStatement = #1AA7EC;
    if (hazardType.equals(kwg_prefix+"Hurricane"))
    {
      colorStatement = #1AA7EC;
      fontSize = 60;
    }
    else if (hazardType.equals(kwg_prefix+"Wildfire"))
    {
      colorStatement = #FF0000;
      fontSize = 30;
    }
    else if (hazardType.equals(kwg_prefix+"EarthquakeEvent"))
    {
      colorStatement = #FFFF00;
      fontSize = 100;
    }
    textSize(fontSize);
    fill(colorStatement);
    text(statement, 0, 0);
    popMatrix();
  }
  
  void update(){
    vel.add(this.gravity);
    pos.add(vel);
    // Hit the ground and splashed
    if(pos.y > height) {
      ss.init(pos.x, height, pos.z, vel.mag(), this.hazard);
      off = true;
    }
  }
}

class RainSystem {
  ArrayList<Droplet> droplets;

  RainSystem() {
    droplets = new ArrayList<Droplet>();
  }

  void run(List <Hazard> hazardStatement_list_sliced) {
    for (int i = 0; i < hazardStatement_list_sliced.size(); i++) {
      droplets.add(new Droplet(random(-width, width*2), random(-300), random(-2500, 500), hazardStatement_list_sliced.get(i)));
    }
    for (int i = droplets.size() - 1; i >= 0; i--) {
      if (droplets.get(i).off) {
        droplets.remove(i);
      } else {
        droplets.get(i).update();
        droplets.get(i).show();
      }
    }
  }
}

class Splash {
  PVector pos;
  PVector vel;
  boolean off;
  float size;
  Hazard hazard;
  PVector gravity;

  Splash(float x, float y, float z, float size, Hazard hazard) {
    this.pos = new PVector(x, y, z);
    this.size = random(0, pow(size,0.15));
    vel = PVector.random3D();
    vel.mult(random(size/4));
    this.hazard = hazard;
    if (this.hazard.hazard_type.equals(kwg_prefix+"EarthquakeEvent"))
    {
      this.gravity = new PVector(0, map(1/hazard.interval, (float) 1/5, (float) 1, 0.9, 1.2), 0);
    }
    else if (this.hazard.hazard_type.equals(kwg_prefix+"Wildfire"))
    {
      this.gravity = new PVector(0, 0.1, 0);
    }
    else if (this.hazard.hazard_type.equals(kwg_prefix+"Hurricane"))
    {
      this.gravity = new PVector(0, map(1/hazard.interval, (float) 1/(24*60), (float) 1/(12*60), 0.2, 0.5), 0);
    }  
  }

  void update() {
    if (pos.y > height) {
      off = true;
      // Only inits another splashes if the size not too small
      if (size > 1) {
        // Hit the ground and splashed
        ss.init(pos.x, height, pos.z, size, this.hazard);
      }
    } else {
      // Otherwise update it
      vel.add(this.gravity);
      pos.add(vel);
    }
  }

  void show() {
    String place = this.hazard.location_label;
    String hazardType = this.hazard.hazard_type;
    strokeWeight(size);
    // The splash is the location of a droplet (hazard)
    pushMatrix();
    translate(pos.x, pos.y, pos.z);
    int fontSize = 30;
    if (hazardType.equals(kwg_prefix+"Hurricane"))
    {
      fontSize = 60;
    }
    else if (hazardType.equals(kwg_prefix+"Wildfire"))
    {
      fontSize = 30;
    }
    else if (hazardType.equals(kwg_prefix+"EarthquakeEvent"))
    {
      fontSize = 100;
    }
    textSize(fontSize);
    fill(#FFFFFF);
    text(place, 0, 0);
    popMatrix();
  }
}

class SplashSystem {
  ArrayList<Splash> splashes;

  SplashSystem() {
    splashes = new ArrayList<Splash>();
  }

  void run() {
    for (int i = splashes.size() - 1; i >= 0; i--) {
      if (splashes.get(i).off) {
        splashes.remove(i);
      } else {
        splashes.get(i).update();
        splashes.get(i).show();
      }
    }
  }
  
  void init(float x, float y, float z, float size, Hazard hazard){
    for (int i = 0; i < size/3; i++) {
      splashes.add(new Splash(x, y, z, size, hazard));
    }
  }
}